@@ -9,9 +9,10 @@ import uuid as uuidgen
 
 # first party
 from ..query import Query
-from ..exception import InterfaceError
+from ..exception import InterfaceError, UniqueError
 from ..decorators import reconnecting
 from ..compat import *
+from ..utils import make_list
 
 
 logger = logging.getLogger(__name__)
@@ -34,7 +35,7 @@ class Connection(object):
         """generate a random transaction name for use in start_transaction() and
         fail_transaction()"""
         name = uuidgen.uuid4()
-        return "p{}".format(str(name.hex))
+        return "p{}".format(str(name)[-5:])
 
     def in_transaction(self):
         """return true if currently in a transaction"""
@@ -63,10 +64,10 @@ class Connection(object):
 
     def _transaction_started(self, name): pass
 
-    def transaction_stop(self):
+    def transaction_stop(self, name):
         """stop/commit a transaction if ready"""
         if self.transaction_count > 0:
-            logger.debug("{}. Stop transaction".format(self.transaction_count))
+            logger.debug("{}. Stop transaction {}".format(self.transaction_count, name))
             if self.transaction_count == 1:
                 self._transaction_stop()
 
@@ -98,10 +99,6 @@ class Connection(object):
 
     def _transaction_failing(self, name): pass
 
-#     def cursor(self, *args, **kwargs):
-#         pout.v("in transaction? {}".format(self.in_transaction()))
-#         return super(Connection, self).cursor(*args, **kwargs)
-
 
 class SQLConnection(Connection):
     def _transaction_start(self):
@@ -139,6 +136,9 @@ class Interface(object):
     connection_config = None
     """a config.Connection() instance"""
 
+    InterfaceError = InterfaceError
+    UniqueError = UniqueError
+
     @classmethod
     def configure(cls, connection_config):
         host = connection_config.host
@@ -259,7 +259,7 @@ class Interface(object):
             connection.transaction_start(name)
             try:
                 yield connection
-                connection.transaction_stop()
+                connection.transaction_stop(name)
 
             except Exception as e:
                 connection.transaction_fail(name)
@@ -426,7 +426,8 @@ class Interface(object):
             except Exception as e:
                 exc_info = sys.exc_info()
                 if self.handle_error(schema, e, **kwargs):
-                    r = self._insert(schema, fields, **kwargs)
+                    with self.transaction(**kwargs):
+                        r = self._insert(schema, fields, **kwargs)
                 else:
                     self.raise_error(e, exc_info)
 
@@ -485,7 +486,7 @@ class Interface(object):
 
             except Exception as e:
                 exc_info = sys.exc_info()
-                if self.handle_error(schema, e, **kwargs):
+                if self.handle_error(schema, e, query=query, **kwargs):
                     ret = callback(schema, query, *args, **kwargs)
                 else:
                     self.raise_error(e, exc_info)
@@ -540,6 +541,18 @@ class Interface(object):
 
     def _delete(self, schema, query, **kwargs): raise NotImplementedError()
 
+    def render(self, schema, query, **kwargs):
+        """Render the query in a way that the interface can interpret it
+
+        so in a SQL interface, this would render SQL, this is mainly used for
+        debugging
+
+        :param query: Query, the Query instance to render
+        :param **kwargs: any named arguments
+        :returns: mixed
+        """
+        raise NotImplementedError()
+
     def spawn(self):
         """Return a new instance of this Interface with the same connection configuration
 
@@ -621,6 +634,7 @@ class SQLInterface(Interface):
         ret = self.query(query_str, *query_args, count_result=True, **kwargs)
         return ret
 
+    # TODO -- rename to execute to match up with cursor interface
     def _query(self, query_str, query_args=None, **query_options):
         """
         **query_options -- dict
@@ -651,11 +665,14 @@ class SQLInterface(Interface):
 
                 elif not ignore_result:
                     if one_result:
-                        ret = self._normalize_result_dict(cur.fetchone())
+                        # https://www.psycopg.org/docs/cursor.html#cursor.fetchone
+                        ret = cur.fetchone()
                     elif count_result:
+                        # https://www.psycopg.org/docs/cursor.html#cursor.rowcount
                         ret = cur.rowcount
                     else:
-                        ret = self._normalize_result_list(cur.fetchall())
+                        # https://www.psycopg.org/docs/cursor.html#cursor.fetchall
+                        ret = cur.fetchall()
 
             except Exception as e:
                 self.log(e)
@@ -663,24 +680,20 @@ class SQLInterface(Interface):
 
             return ret
 
-    def _normalize_result_dict(self, row):
-        return row
-
-    def _normalize_result_list(self, rows):
-        return rows
-
     def _normalize_date_SQL(self, field_name, field_kwargs, symbol):
         raise NotImplemented()
 
     def _normalize_field_SQL(self, schema, field_name, symbol):
         return self._normalize_name(field_name), self.val_placeholder
 
-    def _normalize_val_SQL(self, schema, symbol_map, field_name, field_val, field_kwargs=None):
-
+    def _normalize_val_SQL(self, schema, symbol_map, field):
         format_str = ''
         format_args = []
         symbol = symbol_map['symbol']
-        is_list = symbol_map.get('list', False)
+        is_list = field.is_list
+        field_name = field.name
+        field_val = field.value
+        field_kwargs = field.kwargs
 
         if field_kwargs:
             # kwargs take precedence because None is a perfectly valid field_val
@@ -691,8 +704,13 @@ class SQLInterface(Interface):
                     if format_str:
                         format_str += ' AND '
 
-
                     if is_list:
+                        # you can pass in things like day=..., month=... to
+                        # date fields, this converts those values to lists to
+                        # make sure we can handle something like in_foo(day=1)
+                        # and in_foo(day=[1, 2, 3]) the same way
+                        farg = make_list(farg)
+
                         format_str += '{} {} ({})'.format(
                             fname,
                             symbol,
@@ -709,13 +727,37 @@ class SQLInterface(Interface):
 
         else:
             if is_list and not isinstance(field_val, Query):
+                field_val = make_list(field_val) if field_val else []
                 field_name, format_val_str = self._normalize_field_SQL(schema, field_name, symbol)
-                format_str = '{} {} ({})'.format(
-                    field_name,
-                    symbol,
-                    ', '.join([format_val_str] * len(field_val))
-                )
-                format_args.extend(field_val)
+                if field_val:
+                    format_str = '{} {} ({})'.format(
+                        field_name,
+                        symbol,
+                        ', '.join([format_val_str] * len(field_val))
+                    )
+                    format_args.extend(field_val)
+
+                else:
+                    # field value is empty, so we need to customize the SQL to
+                    # compensate for the empty set since SQL doesn't like empty
+                    # sets
+                    #
+                    # the idea here is this is a condition that will
+                    # automatically cause the query to fail but not necessarily be an error, 
+                    # the best example is the IN (...) queries, if you do self.in_foo([]).get()
+                    # that will fail because the list was empty, but a value error shouldn't
+                    # be raised because a common case is: self.if_foo(Bar.query.is_che(True).pks).get()
+                    # which should result in an empty set if there are no rows where che = TRUE
+                    #
+                    # https://stackoverflow.com/a/58078468/5006
+                    if symbol == "IN":
+                        format_str = '{} <> {}'.format(field_name, field_name)
+
+                    elif symbol == "NOT IN":
+                        format_str = '{} = {}'.format(field_name, field_name)
+
+                    else:
+                        raise ValueError("Unsure what to do here")
 
             else:
                 # special handling for NULL
@@ -774,6 +816,9 @@ class SQLInterface(Interface):
         """
         return '"{}"'.format(name)
 
+    def _normalize_bounds_SQL(self, bounds):
+        raise NotImplemented()
+
     def get_SQL(self, schema, query, **sql_options):
         """
         convert the query instance into SQL
@@ -789,8 +834,8 @@ class SQLInterface(Interface):
         symbol_map = {
             'in': {'symbol': 'IN', 'list': True},
             'nin': {'symbol': 'NOT IN', 'list': True},
-            'is': {'symbol': '=', 'none_symbol': 'IS'},
-            'not': {'symbol': '!=', 'none_symbol': 'IS NOT'},
+            'eq': {'symbol': '=', 'none_symbol': 'IS'},
+            'ne': {'symbol': '!=', 'none_symbol': 'IS NOT'},
             'gt': {'symbol': '>'},
             'gte': {'symbol': '>='},
             'lt': {'symbol': '<'},
@@ -808,9 +853,13 @@ class SQLInterface(Interface):
             query_str.append('SELECT')
             select_fields = query.fields_select
             if select_fields:
-                distinct = "DISTINCT " if select_fields.options.get("unique", False) else ""
-                select_fields_str = distinct + ',{}'.format(os.linesep).join(
-                    (self._normalize_name(f) for f in select_fields.names())
+                distinct_fields = select_fields.options.get(
+                    "distinct",
+                    select_fields.options.get("unique", False)
+                )
+                distinct = "DISTINCT " if distinct_fields else ""
+                select_fields_str = distinct + ",\n".join(
+                    (self._normalize_name(f.name) for f in select_fields)
                 )
             else:
                 select_fields_str = "*"
@@ -832,16 +881,12 @@ class SQLInterface(Interface):
 
                 field_str = ''
                 field_args = []
-                sd = symbol_map[field[0]]
+                sd = symbol_map[field.operator]
 
-                # field[0], field[1], field[2], field[3]
-                _, field_name, field_val, field_kwargs = field
                 field_str, field_args = self._normalize_val_SQL(
                     schema,
                     sd,
-                    field_name,
-                    field_val,
-                    field_kwargs
+                    field,
                 )
 
                 query_str.append('  {}'.format(field_str))
@@ -851,26 +896,21 @@ class SQLInterface(Interface):
             query_sort_str = []
             query_str.append('ORDER BY')
             for field in query.fields_sort:
-                sort_dir_str = 'ASC' if field[0] > 0 else 'DESC'
-                if field[2]:
-                    field_sort_str, field_sort_args = self._normalize_sort_SQL(field[1], field[2], sort_dir_str)
+                sort_dir_str = 'ASC' if field.direction > 0 else 'DESC'
+                if field.value:
+                    field_sort_str, field_sort_args = self._normalize_sort_SQL(field.name, field.value, sort_dir_str)
                     query_sort_str.append(field_sort_str)
                     query_args.extend(field_sort_args)
 
                 else:
-                    query_sort_str.append('  {} {}'.format(field[1], sort_dir_str))
+                    query_sort_str.append('  {} {}'.format(field.name, sort_dir_str))
 
             query_str.append(',{}'.format(os.linesep).join(query_sort_str))
 
         if query.bounds:
-            offset = query.bounds.offset
-            limit = 1 if sql_options.get('one_query', False) else query.bounds.limit
-            query_str.append('LIMIT {} OFFSET {}'.format(
-                limit,
-                offset
-            ))
+            query_str.append(self._normalize_bounds_SQL(query.bounds, sql_options))
 
-        query_str = os.linesep.join(query_str)
+        query_str = "\n".join(query_str)
         return query_str, query_args
 
     def handle_error(self, schema, e, **kwargs):
@@ -878,21 +918,31 @@ class SQLInterface(Interface):
         if not connection: return False
 
         ret = False
-        if connection.closed == 0: # connection is open
-            #connection.transaction_stop()
-            if isinstance(e, InterfaceError):
-                ret = self._handle_error(schema, e.e, **kwargs)
-
-            else:
-                ret = self._handle_error(schema, e, **kwargs)
-
-        else:
+        if connection.closed:
             # we are unsure of the state of everything since this connection has
             # closed, go ahead and close out this interface and allow this query
             # to fail, but subsequent queries should succeed
             self.close()
             ret = True
 
+        else:
+            # connection is open
+            if isinstance(e, InterfaceError):
+                # unwind to the original error
+                while isinstance(e, InterfaceError):
+                    e = e.e
+
+            query = kwargs.get("query", None)
+            schemas = query.schemas if query else []
+            if schemas:
+                ret = True
+                for s in query.schemas:
+                    ret = self._handle_error(s, e, **kwargs)
+                    if not ret:
+                        break
+            else:
+                ret = self._handle_error(schema, e, **kwargs)
+
         return ret
 
     def _handle_error(self, schema, e, **kwargs): raise NotImplemented()
@@ -953,6 +1003,27 @@ class SQLInterface(Interface):
 
         return ret
 
+    def render(self, schema, query, **kwargs):
+        """Render the query
+
+        :param schema: Schema, the query schema
+        :param query: Query, the query to render
+        :param **kwargs: named arguments
+            placeholders: boolean, True if place holders should remain
+        :returns: string if placeholders is False, (string, list) if placeholders is True
+        """
+        sql, sql_args = self.get_SQL(schema, query)
+        placeholders = kwargs.get("placeholders", kwargs.get("placeholder", False))
+
+        if not placeholders:
+            for sql_arg in sql_args:
+                sa = String(sql_arg)
+                if not sa.isnumeric():
+                    sa = "'{}'".format(sa)
+                sql = sql.replace(self.val_placeholder, sa, 1)
+
+        return (sql, sql_args) if placeholders else sql
+
     def _set_all_fields(self, schema, **kwargs):
         """
         this will add fields that don't exist in the table if they can be set to NULL,
